package com.gree.mall.contest.config.aop;

import org.springframework.context.annotation.Bean;
import org.springframework.context.annotation.Configuration;
import org.springframework.context.annotation.EnableAspectJAutoProxy;
import org.springframework.core.annotation.Order;

@Configuration
@EnableAspectJAutoProxy(
        proxyTargetClass = true,  // 强制使用CGLIB代理，确保基于类的代理也能正常工作
        exposeProxy = true        // 暴露代理对象，可通过AopContext获取当前代理
)
public class AopConfig {
    /**
     * 配置说明：
     * 1. @EnableAspectJAutoProxy 启用Spring的AOP自动代理功能
     * 2. proxyTargetClass=true 表示强制使用CGLIB代理：
     *    - 解决JDK动态代理(基于接口)的局限性
     *    - 确保没有实现接口的类也能被代理
     * 3. exposeProxy=true 允许通过AopContext.currentProxy()获取当前代理对象：
     *    - 解决类内部方法调用不走代理的问题
     *    - 使用时需通过 AopContext.currentProxy() 调用方法
     */
    @Bean
    @Order(1)
    public ApiAspect apiAspect() {
        return new ApiAspect();
    }

    @Bean
    @Order(2) // 切面执行顺序，值越小优先级越高
    public OperationLogAspect operationLogAspect() {
        return new OperationLogAspect();
    }


}
