package com.gree.mall.contest.controller.mini.user;

import com.baomidou.mybatisplus.core.metadata.IPage;
import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import com.gree.mall.contest.bean.user.UserAddressBean;
import com.gree.mall.contest.helper.ResponseHelper;
import com.gree.mall.contest.logic.user.AddressLogic;
import com.gree.mall.contest.plus.entity.UserAddress;
import io.swagger.v3.oas.annotations.Operation;
import io.swagger.v3.oas.annotations.Parameter;
import io.swagger.v3.oas.annotations.tags.Tag;
import lombok.RequiredArgsConstructor;
import lombok.extern.slf4j.Slf4j;
import org.springframework.web.bind.annotation.*;

import javax.servlet.http.HttpServletRequest;

@Slf4j
@RestController
@Tag(name = "小程序用户地址", description = "小程序用户地址")
@RequestMapping(value = "/miniapp/user/address", produces = "application/json; charset=utf-8")
@RequiredArgsConstructor
public class AddressController {

    private final AddressLogic addressLogic;

    @PostMapping("/save")
    @Operation(summary = "添加地址")
    public ResponseHelper<UserAddress> saveAddress(
            HttpServletRequest request,
            @Parameter(description = "object",required = true) @RequestBody UserAddressBean userAddressBean
    ) throws Exception{
        UserAddress userAddress = addressLogic.saveAddress(request , userAddressBean);
        return ResponseHelper.success(userAddress);
    }

    @PostMapping("/update")
    @Operation(summary = "修改地址")
    public ResponseHelper<UserAddress> updateAddress(
            HttpServletRequest request,
            @Parameter(description = "object",required = true) @RequestBody UserAddressBean  userAddressBean
    ) throws Exception{
        addressLogic.updateAddress(request, userAddressBean);
        return ResponseHelper.success();
    }

    @PostMapping("/del")
    @Operation(summary = "删除地址")
    public ResponseHelper<String> delAddress(
            @Parameter(description = "用户地址id", required = true) @RequestParam(required = true) String userAddressId
    ) throws Exception{
        String addressId = addressLogic.delAddress(userAddressId);
        return ResponseHelper.success(addressId);
    }
    @GetMapping("/detail")
    @Operation(summary = "获取地址详情")
    public ResponseHelper<UserAddress> addressDetail(
            @Parameter(description = "用户地址id", required = true) @RequestParam(required = true) String userAddressId
    ) throws Exception{
        UserAddress userAddress = addressLogic.addressDetail(userAddressId);
        return ResponseHelper.success(userAddress);
    }


    @GetMapping("/list")
    @Operation(summary = "地址列表")
    public ResponseHelper<Page<UserAddress>> pageAddress(
            @Parameter(description = "用户id", required = true) @RequestParam(required = true) String userId,
            @Parameter(description = "页号",required = true) @RequestParam Integer pageNum,
            @Parameter(description = "页大小",required = true) @RequestParam Integer pageSize
    ) throws Exception{

        IPage<UserAddress> page = addressLogic.page(userId,pageNum,pageSize);
        return ResponseHelper.success(page);
    }


}
