package com.gree.mall.contest.controller.pc.admin;

import com.baomidou.mybatisplus.core.metadata.IPage;
import com.gree.mall.contest.exception.RemoteServiceException;
import com.gree.mall.contest.helper.ResponseHelper;
import com.gree.mall.contest.logic.admin.OperationLogLogic;
import com.gree.mall.contest.plus.entity.OperationLog;
import io.swagger.v3.oas.annotations.Operation;
import io.swagger.v3.oas.annotations.Parameter;
import io.swagger.v3.oas.annotations.tags.Tag;
import lombok.RequiredArgsConstructor;
import lombok.extern.slf4j.Slf4j;
import org.springframework.web.bind.annotation.GetMapping;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RequestParam;
import org.springframework.web.bind.annotation.RestController;

import javax.servlet.http.HttpServletRequest;
import java.util.List;

@Slf4j
@RestController
@Tag(name = "操作日志", description = "操作日志" )
@RequestMapping(value = "/pc/admin/operation/log", produces = "application/json; charset=utf-8")
@RequiredArgsConstructor
public class OperationLogController {
    
    private final OperationLogLogic operationLogLogic;

    @GetMapping("/list")
    @Operation(summary = "列表")
    public ResponseHelper<IPage<OperationLog>> list(
            HttpServletRequest request,
            @Parameter(description = "模块名称",required = false) @RequestParam(required = false) String moduleName,
            @Parameter(description  = "商户名称",required = false) @RequestParam(required = false) String companyWechatName,
            @Parameter(description  = "账号昵称",required = false) @RequestParam(required = false) String nickName,
            @Parameter(description  = "开始时间",required = false) @RequestParam(required = false) String startTime,
            @Parameter(description  = "结束时间",required = false) @RequestParam(required = false) String endTime,
            @Parameter(description  = "页号",required = true) @RequestParam(required = true) Integer pageNo,
            @Parameter(description  = "页大小",required = true) @RequestParam(required = true) Integer pageSize
    ) throws RemoteServiceException {
        IPage<OperationLog> operationLogIPage = operationLogLogic.listPage(request, moduleName,companyWechatName,nickName, startTime, endTime, pageNo, pageSize);
        return ResponseHelper.success(operationLogIPage);
    }

    @GetMapping("/module")
    @Operation(summary = "所属模块")
    public ResponseHelper<List<String>> moduleList(HttpServletRequest request){
        return ResponseHelper.success(operationLogLogic.moduleList(request));
    }


    @GetMapping("/detail")
    @Operation(summary = "详情")
    public ResponseHelper<OperationLog> detail(
            @Parameter(description  = "operationLogId",required = true) @RequestParam(required = true) String operationLogId
    ) throws RemoteServiceException {
        OperationLog detail = operationLogLogic.detail(operationLogId);
        return ResponseHelper.success(detail);
    }
}
