package com.gree.mall.contest.logic;

import cn.hutool.core.collection.CollectionUtil;
import com.aliyuncs.utils.StringUtils;
import com.baomidou.mybatisplus.core.metadata.IPage;
import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import com.gree.mall.contest.bean.admin.AdminUserCom;
import com.gree.mall.contest.exception.RemoteServiceException;
import com.gree.mall.contest.logic.common.CommonLogic;
import com.gree.mall.contest.plus.entity.Storage;
import com.gree.mall.contest.plus.service.StorageService;
import lombok.RequiredArgsConstructor;
import lombok.extern.slf4j.Slf4j;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;

import javax.servlet.http.HttpServletRequest;
import java.util.Date;

@Service
@Slf4j
@RequiredArgsConstructor
public class StorageLogic {


    private final CommonLogic commonLogic;
    private final StorageService storageService;

    /**
     * 仓储列表
     *
     * @param pageNum
     * @param pageSize
     * @return
     */
    public IPage<Storage> page(Integer pageNum, Integer pageSize, HttpServletRequest request) {
        AdminUserCom adminUser = commonLogic.getAdminUser(request);

        return storageService.lambdaQuery()
                .in(CollectionUtil.isNotEmpty(adminUser.getCompanyWechatIds()),Storage::getCompanyWechatId,adminUser.getCompanyWechatIds())
                .page(new Page<>(pageNum, pageSize));
    }

    /**
     * 仓储详情
     *
     * @param id
     * @return
     */
    public Storage detail(String id) {
        return storageService.getById(id);
    }

    public Storage detailByName(String companyWechatId,String name) {
        return storageService
                .lambdaQuery()
                .eq(Storage::getCompanyWechatId, companyWechatId)
                .eq(Storage::getStorageName,name)
                .one();
    }

    /**
     * 新增仓储
     *
     * @param storage
     */
    @Transactional(rollbackFor = Exception.class)
    public void add(HttpServletRequest request,Storage storage) {
        AdminUserCom adminUser = commonLogic.getAdminUser(request);
        if(StringUtils.isEmpty(adminUser.getLoginCompanyWechatId())){
            throw new RemoteServiceException("无效企业微信id");
        }
        storage.setCreateTime(new Date());
        storage.setCompanyWechatId(adminUser.getLoginCompanyWechatId());
        storage.setCompanyName(adminUser.getLoginCompanyName());
        storageService.save(storage);
    }

    /**
     * 修改仓储
     *
     * @param storage
     */
    @Transactional(rollbackFor = Exception.class)
    public void update(Storage storage) {
        storage.setCreateTime(null);
        storageService.updateById(storage);
    }

    /**
     * 删除仓储
     *
     * @param storageId
     */
    @Transactional(rollbackFor = Exception.class)
    public void delete(String storageId) {
        storageService.removeById(storageId);
    }
}
