package com.gree.mall.contest.logic.goods;

import com.gree.mall.contest.bean.admin.AdminUserCom;
import com.gree.mall.contest.bean.goods.GoodsCategoryBean;
import com.gree.mall.contest.commonmapper.CustomGoodsCategoryMapper;
import com.gree.mall.contest.exception.RemoteServiceException;
import com.gree.mall.contest.logic.common.CommonLogic;
import com.gree.mall.contest.plus.entity.GoodsCategory;
import com.gree.mall.contest.plus.service.GoodsCategoryService;
import lombok.RequiredArgsConstructor;
import lombok.extern.slf4j.Slf4j;
import org.apache.commons.lang3.StringUtils;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;

import javax.servlet.http.HttpServletRequest;
import java.util.Date;
import java.util.List;
import java.util.stream.Collectors;

@Service
@Slf4j
@RequiredArgsConstructor
public class GoodsCategoryLogic {

    private static final Logger operationLogger = LoggerFactory.getLogger("OPERATION_LOGGER");

    private final CustomGoodsCategoryMapper customGoodsCategoryMapper;
    private final GoodsCategoryService goodsCategoryService;
    private final CommonLogic commonLogic;

    /**
     * 分类管理列表
     *
     * @param level
     * @param status
     * @return
     */
    public List<GoodsCategoryBean> list(Integer level, String name, Boolean status, HttpServletRequest request) {
        //获取当前登录企业微信id
        AdminUserCom adminUser = commonLogic.getAdminUser(request);
        List<String> companyWechatIds = adminUser.getCompanyWechatIds();


        return customGoodsCategoryMapper.list(level, status, name, companyWechatIds);
    }

    /**
     * 新增分类
     *
     * @param goodsCategoryBean
     */
    @Transactional(rollbackFor = Exception.class)
    public void add(GoodsCategoryBean goodsCategoryBean, HttpServletRequest request) {
        AdminUserCom adminUser = commonLogic.getAdminUser(request);
        if (StringUtils.isEmpty(adminUser.getAdminCompanyWechat().getCompanyWechatId())) {
            throw new RemoteServiceException("无效微信企业id");
        }

        Date createDate = new Date();
        if (goodsCategoryBean.getChildren() != null) {
            List<GoodsCategory> goodsCategoryList = goodsCategoryBean.getChildren()
                    .stream()
                    .map(c -> c.setCreateTime(createDate))
                    .collect(Collectors.toList());
            for (GoodsCategory goodsCategory : goodsCategoryList) {
                goodsCategory.setLevel(2);
                goodsCategory.setCreateTime(new Date());
                goodsCategory.setCompanyWechatId(adminUser.getAdminCompanyWechat().getCompanyWechatId());
                goodsCategory.setCompanyName(adminUser.getAdminCompanyWechat().getCompanyName());
            }
            goodsCategoryService.saveBatch(goodsCategoryList);
        } else {
            goodsCategoryBean.setCreateTime(createDate);
            goodsCategoryBean.setCompanyWechatId(adminUser.getAdminCompanyWechat().getCompanyWechatId());
            goodsCategoryBean.setCompanyName(adminUser.getAdminCompanyWechat().getCompanyName());
            goodsCategoryService.save(goodsCategoryBean);
        }
    }

    /**
     * 修改分类
     *
     * @param goodsCategory
     */
    @Transactional(rollbackFor = Exception.class)
    public void update(GoodsCategory goodsCategory) {
        goodsCategoryService.updateById(goodsCategory);
    }

    /**
     * 删除分类
     *
     * @param categoryId
     */
    @Transactional(rollbackFor = Exception.class)
    public void delete(String categoryId) {
        // 删除子类别
        goodsCategoryService.lambdaUpdate()
                .eq(GoodsCategory::getCategoryId, categoryId)
                .or()
                .eq(GoodsCategory::getParentId, categoryId)
                .remove();
    }

    /**
     * 分类详情
     */
    public GoodsCategory detail(String goodsCategoryId) {
        return goodsCategoryService.getById(goodsCategoryId);
    }

}
