package com.gree.mall.contest.logic.notice;

import cn.hutool.core.collection.CollectionUtil;
import com.aliyuncs.utils.StringUtils;
import com.baomidou.mybatisplus.core.metadata.IPage;
import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import com.gree.mall.contest.bean.admin.AdminUserCom;
import com.gree.mall.contest.exception.RemoteServiceException;
import com.gree.mall.contest.logic.common.CommonLogic;
import com.gree.mall.contest.plus.entity.GoodsSpec;
import com.gree.mall.contest.plus.entity.Notice;
import com.gree.mall.contest.plus.entity.PromotionGroupSpec;
import com.gree.mall.contest.plus.service.GoodsSpecService;
import com.gree.mall.contest.plus.service.NoticeService;
import com.gree.mall.contest.plus.service.PromotionGroupSpecService;
import lombok.RequiredArgsConstructor;
import lombok.extern.slf4j.Slf4j;
import org.springframework.stereotype.Service;

import javax.servlet.http.HttpServletRequest;
import java.util.*;

@Service
@Slf4j
@RequiredArgsConstructor
public class NoticeLogic {

    private final NoticeService noticeService;
    private final GoodsSpecService goodsSpecService;
    private final PromotionGroupSpecService promotionGroupSpecService;
    private final CommonLogic commonLogic;

    private class Node implements Map.Entry<String,String>{

        String key;
        String value;

        public Node(String key,String value){
            this.key=key;
            this.value=value;
        }

        @Override
        public String getKey() {
            return key;
        }

        @Override
        public String getValue() {
            return value;
        }

        @Override
        public String setValue(String value) {
            String oldValue = this.value;
            this.value = value;
            return oldValue;
        }
    }
    /**
     * 商品库存提醒
     */
    public void stockNotice(){
        final int stockNum = 10;
        //基础商品
        List<GoodsSpec> list = goodsSpecService.lambdaQuery()
                .le(GoodsSpec::getStockNum, stockNum)
                .list();

        Map<String,Node> goodsIdMap=new HashMap<>();
        for (GoodsSpec goodsSpec:list
             ) {
            Node node = new Node( goodsSpec.getCompanyWechatId() , goodsSpec.getCompanyName() );
            goodsIdMap.put( goodsSpec.getGoodsId() , node );
        }
        //新增基础商品通知
        this.addNotice(goodsIdMap,2);


        goodsIdMap=new HashMap<>();


        //团购商品
        List<PromotionGroupSpec> list3 = promotionGroupSpecService.lambdaQuery()
                .le(PromotionGroupSpec::getStock, stockNum)
                .list();
        goodsIdMap=new HashMap<>();
        for (PromotionGroupSpec promotionGroupSpec:list3
             ) {
            Node node=new Node(promotionGroupSpec.getCompanyWechatId() , promotionGroupSpec.getCompanyName());
            goodsIdMap.put(promotionGroupSpec.getGoodsId(),node);
        }
        //新增团购商品通知
        this.addNotice(goodsIdMap,4);

    }

    private void addNotice(Map<String,Node> goodsIdMap,Integer type){
        if(goodsIdMap.size() == 0){
            return;
        }
        Map<String,String> map = new HashMap<>();
        map.put("2","基础商品");
        map.put("4","团购商品");

        List<Notice> notices = new ArrayList<>();
        Date date = new Date();
        for(String goodsId : goodsIdMap.keySet()) {
            Notice notice = new Notice();
            notice.setObjId(goodsId);
            notice.setTitle("商品库存");
            notice.setType(type);
            notice.setContent(map.get(type.toString())+"id:"+goodsId+",商品库存不足10,请及时补充库存");
            notice.setCreateTime(date);
            notice.setCompanyWechatId(goodsIdMap.get(goodsId).getKey());
            notice.setCompanyName(goodsIdMap.get(goodsId).getValue());
            notices.add(notice);
        }
        noticeService.saveBatch(notices);
    }


    public IPage<Notice> page(HttpServletRequest request,Boolean readFlag, Integer pageNum, Integer pageSize) {
       /* List<String> websitNoList = new ArrayList<>();
        AdminWebsitBean adminWebsitBean = CommonUtils.getAdminUser(request);
        if (!adminWebsitBean.getType()) {
            websitNoList = adminWebsitBean.getWeidsitIds().stream().map(WebsitZone::getWebsitNumber).collect(Collectors.toList());
        }*/

        //获取当前登录企业微信id
        AdminUserCom adminUser = commonLogic.getAdminUser(request);
        List<String> companyWechatIds = adminUser.getCompanyWechatIds();

        return noticeService.lambdaQuery()
               // .in(!CollectionUtils.isEmpty(websitNoList),Notice::getWebsitNumber,websitNoList)
                .in(CollectionUtil.isNotEmpty(companyWechatIds),Notice::getCompanyWechatId,companyWechatIds)
                .eq(readFlag!=null,Notice::getReadFlag,readFlag)
                .orderByDesc(Notice::getCreateTime)
                .page(new Page(pageNum, pageSize));
    }

    public Long unreadCount(HttpServletRequest request) {
       /* List<String> websitNoList = new ArrayList<>();
        AdminWebsitBean adminWebsitBean = CommonUtils.getAdminUser(request);
        if(adminWebsitBean == null){
            return 0;
        }
        if (!adminWebsitBean.getType()) {
            websitNoList = adminWebsitBean.getWeidsitIds().stream().map(WebsitZone::getWebsitNumber).collect(Collectors.toList());
        }*/
        //获取当前登录企业微信id
        AdminUserCom adminUser = commonLogic.getAdminUser(request);
        List<String> companyWechatIds = adminUser.getCompanyWechatIds();

        return noticeService.lambdaQuery()
                //.in(!adminWebsitBean.getType(),Notice::getWebsitNumber,websitNoList)
                .in(CollectionUtil.isNotEmpty(companyWechatIds) , Notice::getCompanyWechatId , companyWechatIds)
                .eq(Notice::getReadFlag,false)
                .count();
    }

    public void markRead(String[] noticeIds) {

        for(String noticeId:noticeIds){
            noticeService.lambdaUpdate()
                    .eq(Notice::getNoticeId,noticeId)
                    .set(Notice::getReadFlag,true)
                    .update();
        }
    }

    public void markReadAll(HttpServletRequest request) {
        /*List<String> websitNoList = new ArrayList<>();
        AdminWebsitBean adminWebsitBean = CommonUtils.getAdminUser(request);
        if (!adminWebsitBean.getType()) {
            websitNoList = adminWebsitBean.getWeidsitIds().stream().map(WebsitZone::getWebsitNumber).collect(Collectors.toList());
        }*/
        AdminUserCom adminUser = commonLogic.getAdminUser(request);
        String companyWechatId = adminUser.getLoginCompanyWechatId();
        if(StringUtils.isEmpty(companyWechatId)){
            throw new RemoteServiceException("无效微信企业id");
        }

        noticeService.lambdaUpdate()
                //.in(!adminWebsitBean.getType(),Notice::getWebsitNumber,websitNoList)
                .eq(Notice::getCompanyWechatId , companyWechatId)
                .eq(Notice::getReadFlag,false)
                .set(Notice::getReadFlag,true)
                .update();
    }

}
