package com.gree.mall.contest.logic.user;

import com.baomidou.mybatisplus.core.metadata.IPage;
import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import com.gree.mall.contest.bean.user.CurrentCompanyWechat;
import com.gree.mall.contest.bean.user.UserAddressBean;
import com.gree.mall.contest.logic.common.CommonLogic;
import com.gree.mall.contest.plus.entity.FreightTemplateDetail;
import com.gree.mall.contest.plus.entity.UserAddress;
import com.gree.mall.contest.plus.service.FreightTemplateDetailService;
import com.gree.mall.contest.plus.service.UserAddressService;
import lombok.RequiredArgsConstructor;
import lombok.extern.slf4j.Slf4j;
import org.apache.commons.lang3.StringUtils;
import org.springframework.beans.BeanUtils;
import org.springframework.stereotype.Service;

import javax.servlet.http.HttpServletRequest;
import java.util.Date;
import java.util.Objects;

@Service
@Slf4j
@RequiredArgsConstructor
public class AddressLogic {

    private final FreightTemplateDetailService freightTemplateDetailService;
    private final UserAddressService userAddressService;
    private final CommonLogic commonLogic;

    public UserAddress saveAddress(HttpServletRequest request, UserAddressBean userAddressBean) {
        CurrentCompanyWechat currentCompanyWechat = commonLogic.getCurrentCompanyWechat(request);

        UserAddress userAddress = new UserAddress();
        BeanUtils.copyProperties(userAddressBean, userAddress);
        userAddress.setUserAddressId(null);
        Long count = freightTemplateDetailService.lambdaQuery()
                .eq(StringUtils.isNotBlank(userAddressBean.getProvince()), FreightTemplateDetail::getProvince, userAddressBean.getProvince())
                .eq(StringUtils.isNotBlank(userAddressBean.getCity()), FreightTemplateDetail::getCity, userAddressBean.getCity())
                .eq(FreightTemplateDetail::getCompanyWechatId , currentCompanyWechat.getCurrentCompanyWechatId())
                .count();
        userAddress.setStatus(count > 0);
        userAddress.setCreateTime(new Date());
        if (userAddressBean.getDefaultAddr() != null && userAddressBean.getDefaultAddr()) {
            userAddressService.lambdaUpdate().eq(UserAddress::getUserId, userAddressBean.getUserId())
                    .set(UserAddress::getDefaultAddr, false)
                    .update();
        }
        userAddress.setCompanyWechatId(currentCompanyWechat.getCurrentCompanyWechatId());
        userAddress.setCompanyName(currentCompanyWechat.getCurrentCompanyName());
        userAddressService.save(userAddress);
        return userAddress;
    }

    public void updateAddress(HttpServletRequest request , UserAddressBean userAddressBean) {
        CurrentCompanyWechat currentCompanyWechat = commonLogic.getCurrentCompanyWechat(request);

        BeanUtils.copyProperties(userAddressBean, userAddressBean);
        Long count = freightTemplateDetailService.lambdaQuery()
                .eq(StringUtils.isNotBlank(userAddressBean.getProvince()), FreightTemplateDetail::getProvince, userAddressBean.getProvince())
                .eq(StringUtils.isNotBlank(userAddressBean.getCity()), FreightTemplateDetail::getCity, userAddressBean.getCity())
                .eq(FreightTemplateDetail::getCompanyWechatId , currentCompanyWechat.getCurrentCompanyWechatId())
                .count();
        boolean status = count > 0;
        if (userAddressBean.getDefaultAddr() != null && userAddressBean.getDefaultAddr()) {
            userAddressService.lambdaUpdate().eq(UserAddress::getUserId, userAddressBean.getUserId())
                    .set(UserAddress::getDefaultAddr, false)
                    .update();
        }
        userAddressService.lambdaUpdate().eq(UserAddress::getUserAddressId, userAddressBean.getUserAddressId())
                .set(StringUtils.isNotBlank(userAddressBean.getAddress()), UserAddress::getAddress, userAddressBean.getAddress())
                .set(StringUtils.isNotBlank(userAddressBean.getProvince()), UserAddress::getProvince, userAddressBean.getProvince())
                .set(StringUtils.isNotBlank(userAddressBean.getCity()), UserAddress::getCity, userAddressBean.getCity())
                .set(StringUtils.isNotBlank(userAddressBean.getArea()), UserAddress::getArea, userAddressBean.getArea())
                .set(StringUtils.isNotBlank(userAddressBean.getHouseNo()), UserAddress::getHouseNo, userAddressBean.getHouseNo())
                .set(StringUtils.isNotBlank(userAddressBean.getName()), UserAddress::getName, userAddressBean.getName())
                .set(StringUtils.isNotBlank(userAddressBean.getPhone()), UserAddress::getPhone, userAddressBean.getPhone())
                .set(UserAddress::getStatus, status)
                .set(StringUtils.isNotEmpty(userAddressBean.getStreet()), UserAddress::getStreet, userAddressBean.getStreet())
                .set(!Objects.isNull(userAddressBean.getDefaultAddr()), UserAddress::getDefaultAddr, userAddressBean.getDefaultAddr())
                .update();
    }


    public String delAddress(String userAddressId) {

        userAddressService.lambdaUpdate().eq(UserAddress::getUserAddressId, userAddressId).remove();
        return userAddressId;
    }


    public UserAddress addressDetail(String userAddressId) {

        UserAddress userAddress = userAddressService.lambdaQuery().eq(UserAddress::getUserAddressId, userAddressId).one();
        return userAddress;
    }

    public IPage<UserAddress> page(String userId, Integer pageNum, Integer pageSize) {
        return userAddressService.lambdaQuery()
                .eq(StringUtils.isNotBlank(userId), UserAddress::getUserId, userId)
                .orderByDesc(UserAddress::getCreateTime)
                .page(new Page(pageNum, pageSize));

    }


}


