package com.gree.mall.contest.logic.user;

import cn.binarywang.wx.miniapp.api.WxMaService;
import cn.binarywang.wx.miniapp.bean.WxMaPhoneNumberInfo;
import cn.hutool.core.collection.CollectionUtil;
import cn.hutool.core.date.DateTime;
import cn.hutool.core.date.DateUtil;
import cn.hutool.core.util.IdUtil;
import cn.hutool.json.JSONArray;
import cn.hutool.json.JSONUtil;
import com.baomidou.mybatisplus.core.conditions.query.LambdaQueryWrapper;
import com.baomidou.mybatisplus.core.metadata.IPage;
import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import com.gree.mall.contest.bean.ExcelData;
import com.gree.mall.contest.bean.admin.AdminUserCom;
import com.gree.mall.contest.bean.common.WechatOpenBean;
import com.gree.mall.contest.bean.member.MemberBean;
import com.gree.mall.contest.bean.member.MemberSortBean;
import com.gree.mall.contest.bean.user.*;
import com.gree.mall.contest.commonmapper.CustomGoodsMapper;
import com.gree.mall.contest.commonmapper.CustomMemberMapper;
import com.gree.mall.contest.commonmapper.OrderShareRankMapper;
import com.gree.mall.contest.constant.Constant;
import com.gree.mall.contest.enums.OrderShareStatusEnum;
import com.gree.mall.contest.enums.QrCodeEnum;
import com.gree.mall.contest.enums.UserTypeEnum;
import com.gree.mall.contest.exception.RemoteServiceException;
import com.gree.mall.contest.helper.ResponseHelper;
import com.gree.mall.contest.logic.activity.PromotionGroupLogic;
import com.gree.mall.contest.logic.common.CommonLogic;
import com.gree.mall.contest.logic.common.WechatLogic;
import com.gree.mall.contest.logic.coupon.CouponLogic;
import com.gree.mall.contest.logic.order.OrderShareLogic;
import com.gree.mall.contest.logic.order.ShoppingCartLogic;
import com.gree.mall.contest.plus.entity.*;
import com.gree.mall.contest.plus.service.*;
import com.gree.mall.contest.utils.CommonUtils;
import com.gree.mall.contest.utils.JwtUtils;
import com.gree.mall.contest.utils.RedisUtil;
import com.gree.mall.contest.utils.excel.ExcelUtils;
import lombok.RequiredArgsConstructor;
import lombok.extern.slf4j.Slf4j;
import org.apache.commons.collections4.CollectionUtils;
import org.apache.commons.lang3.StringEscapeUtils;
import org.apache.commons.lang3.StringUtils;
import org.springframework.beans.BeanUtils;
import org.springframework.integration.redis.util.RedisLockRegistry;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;
import org.springframework.web.multipart.MultipartFile;

import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;
import java.io.IOException;
import java.math.BigDecimal;
import java.text.SimpleDateFormat;
import java.util.*;
import java.util.concurrent.TimeUnit;
import java.util.concurrent.locks.Lock;
import java.util.function.Function;
import java.util.stream.Collectors;

/**
 * 小程序用户
 */
@Service
@Slf4j
@RequiredArgsConstructor
public class UserLogic {

    private final UserService userService;
    private final UserAddressService userAddressService;
    private final GoodsVisitService goodsVisitService;
    private final CustomGoodsMapper customGoodsMapper;
    private final CustomMemberMapper customMemberMapper;
    private final CouponLogic couponLogic;
    private final OrderShareLogic orderShareLogic;
    private final WechatLogic wechatLogic;
    private final PromotionGroupLogic promotionGroupLogic;
    private final CommonLogic commonLogic;
    private final UserServiceApplyService userServiceApplyService;
    private final UserServiceWaitService userServiceWaitService;
    private final AdminWebsitService adminWebsitService;
    private final OrderShareRankMapper orderShareRankMapper;
    private final RedisLockRegistry redisLockRegistry;
    private final ShoppingCartLogic shoppingCartLogic;
    private final OrderShareService orderShareService;
    private final RedisUtil redisUtil;

    /**
     * 会员列表
     *
     * @param keyword
     * @param startTime
     * @param endTime
     * @param pageNum
     * @param pageSize
     * @return
     */
    public IPage<User> page(HttpServletRequest request, String keyword, String type, Boolean promotionGroupLeader,
                            Boolean innerr, String startTime, String endTime, String sortJson, String websitId, Boolean isExchangeCode,
                            Integer pageNum, Integer pageSize) {

        //获取当前登录企业微信id
        AdminUserCom adminUser = commonLogic.getAdminUser(request);
        List<String> companyWechatIds = adminUser.getCompanyWechatIds();

        LambdaQueryWrapper<User> memberLambdaQueryWrapper = new LambdaQueryWrapper<>();
        memberLambdaQueryWrapper
                .in(CollectionUtils.isNotEmpty(companyWechatIds), User::getCompanyWechatId, companyWechatIds)
                .eq(isExchangeCode != null, User::getIsExchangeCode, isExchangeCode)
                .eq(promotionGroupLeader != null, User::getPromotionGroupLeader, promotionGroupLeader)
                .eq(StringUtils.isNotEmpty(type), User::getType, type)
                .eq(innerr != null, User::getInnerr, innerr)
                // .in(!(adminWebsitBean.getType() && org.apache.commons.lang3.StringUtils.isEmpty(websitId)), User::getWebsitNumber, websitNoList)
                .between(StringUtils.isNotBlank(startTime) && StringUtils.isNotBlank(endTime), User::getCreateTime, startTime, endTime)
                .and(v -> v.ne(User::getMobile, "").or().ne(User::getWorkPhone, ""))
                .and(StringUtils.isNotBlank(keyword),
                        memberLambdaQueryWrapper1 -> memberLambdaQueryWrapper1.like(User::getNickName, keyword)
                                .or()
                                .like(User::getMobile, keyword));
        if (sortJson != null && !sortJson.trim().equals("")) {
            JSONArray goodsSortBeanArray = JSONUtil.parseArray(StringEscapeUtils.unescapeJava(sortJson));
            MemberSortBean.convertColumn(goodsSortBeanArray, memberLambdaQueryWrapper);
        } else {
            memberLambdaQueryWrapper.orderByDesc(User::getCreateTime);
        }

        return userService.page(new Page(pageNum, pageSize), memberLambdaQueryWrapper);
    }


    public void export(HttpServletRequest request, HttpServletResponse response, String keyword, String type, String websitId, String startTime, String endTime) throws Exception {

        //获取当前登录企业微信id
        AdminUserCom adminUser = commonLogic.getAdminUser(request);
        List<String> companyWechatIds = adminUser.getCompanyWechatIds();

        List<MemberBean> userList = customMemberMapper.queryUser(type, keyword, null, startTime, endTime, companyWechatIds);

        log.info("会员导出请求参数{},{},{},{}", type, keyword, startTime, endTime);
        log.info("会员导出数据条数{}", userList == null ? 0 : userList.size());
        String[] titles = {"序号", "微信手机号", "微信昵称", "企微名称", "企微手机号", "用户类型", "所属网点", "成为业务员时间", "所在地区"};
        ExcelData excelData = new ExcelData();
        List<List<Object>> rows = new ArrayList<>();
        SimpleDateFormat simpleDateFormat = new SimpleDateFormat("yyyy-MM-dd HH:mm:ss");
        for (int i = 0; i < userList.size(); i++) {
            MemberBean bean = userList.get(i);
            List<Object> row = new ArrayList<>();
            row.add(i + 1);
            row.add(bean.getMobile());
            row.add(bean.getNickName());
            row.add(bean.getWorkName());
            row.add(bean.getWorkPhone());
            row.add(bean.getType().equals(UserTypeEnum.GENERAL.toString()) ? "普通用户" : "业务员");
            row.add("");
            row.add(bean.getCreateTime() == null ? "" : simpleDateFormat.format(bean.getCreateTime()));
            String city = "";
            row.add(city);
            rows.add(row);
        }
        String fileName = String.format("用户信息导出_%s.xlsx", simpleDateFormat.format(new Date()));
        excelData.setTitles(Arrays.asList(titles));
        excelData.setName("用户信息");
        excelData.setRows(rows);
        ExcelUtils.exportExcel(request, response, fileName, excelData, -1, null);
    }

    /**
     * 更改会员状态
     *
     * @param userId
     * @param status
     */
    public void updateState(String userId, Boolean status) {
        userService.lambdaUpdate()
                .eq(User::getUserId, userId)
                .set(User::getStatus, status)
                .update();
    }


    /**
     * 会员资料修改
     *
     * @param user
     */
    public void update(User user) {
        user.updateById();
    }

    /**
     * 修改会员业务状态
     */
    @Transactional
    public void changeServiceUser(String userId, String websitId, String websitName, String workUserId, String workName,
                                  String position, String mobile, Boolean isService) {
        User user = userService.getById(userId);
        if (user.getInnerr()) {
            throw new RemoteServiceException("请先取消内部人员");
        }

        final List<String> userIds = Collections.singletonList(userId);
        this.batchChangeServiceUser(userIds, isService);

        if (isService) {
            if (StringUtils.isBlank(workUserId) || StringUtils.isBlank(workName) || StringUtils.isBlank(position) ||
                    StringUtils.isBlank(mobile) || StringUtils.isBlank(websitId) || StringUtils.isBlank(websitName)) {
                throw new RemoteServiceException("请填写关键信息");
            }
            userService.lambdaUpdate()
                    .set(User::getWorkUserId, workUserId)
                    .set(User::getWebsitId, websitId)
                    .set(User::getWebsitName, websitName)
                    .set(User::getWorkName, workName)
                    .set(User::getPosition, position)
                    .set(User::getWorkPhone, mobile)
                    .set(User::getWorkCreateTime, new Date())
                    .eq(User::getUserId, user.getUserId())
                    .update();
        }
    }

    private void batchChangeServiceUser(List<String> userIds, Boolean isService) {
        // 1.将用户设为业务员或非业务员
        userService.lambdaUpdate()
                .set(User::getType, isService ? UserTypeEnum.SERVICE.toString() : UserTypeEnum.GENERAL.toString())
                .set(isService, User::getWorkCreateTime, DateUtil.date())
                .set(!isService, User::getWorkCreateTime, null)
                .in(User::getUserId, userIds)
                .update();

        // 2.启用或禁用 用户分享优惠券
        couponLogic.updateUserCouponTransfer(userIds, isService);
    }

    /**
     * 修改会员内部人员状态
     */
    public void updateUserInnerStatus(String userId, Boolean inner) {
        Long count = userService.lambdaQuery()
                .eq(User::getUserId, userId)
                .eq(User::getType, UserTypeEnum.SERVICE.getKey())
                .count();
        if (count == 0) {
            throw new RemoteServiceException("只有业务员才能成为内部人员");
        }
        userService.lambdaUpdate()
                .set(User::getInnerr, inner)
                .eq(User::getUserId, userId)
                .eq(User::getType, UserTypeEnum.SERVICE)
                .update();
    }

    /**
     * 导入跟新内部人员状态
     */
    @Transactional
    public void localUpdateInner(HttpServletRequest request, MultipartFile multipartFile) throws IOException {
        final AdminUserCom adminUser = commonLogic.getAdminUser(request);
        final AdminCompanyWechat companyWechat = adminUser.getAdminCompanyWechat();
        String loginCompanyWechatId = adminUser.getLoginCompanyWechatId();
        if (StringUtils.isEmpty(loginCompanyWechatId)) {
            throw new RemoteServiceException("无效企业微信id");
        }

        List<Object> objects = ExcelUtils.importExcel(multipartFile);
        List<User> phoneList = new ArrayList<>();
        List<User> innerPhoneList = new ArrayList<>();
        for (int i = 0; i < objects.size(); i++) {
            List<String> rows = (List<String>) objects.get(i);
            CommonUtils.initList(rows, 7);
            String typeStr = Objects.isNull(rows.get(1)) ? "业务员" : rows.get(1);
            boolean isInner = typeStr.equals("内部人员");
            User user = new User();
            user.setMobile(rows.get(0));
            user.setWebsitId(StringUtils.isNotBlank(rows.get(2)) ? rows.get(2).trim() : null);
            user.setWebsitName(StringUtils.isNotBlank(rows.get(3)) ? rows.get(3).trim() : null);
            user.setPosition(StringUtils.isNotBlank(rows.get(4)) ? rows.get(4).trim() : null);
            user.setWorkName(StringUtils.isNotBlank(rows.get(5)) ? rows.get(5).trim() : null);
            user.setWorkUserId(StringUtils.isNotBlank(rows.get(6)) ? rows.get(6).trim() : null);
            user.setWorkPhone(user.getMobile());
            user.setInnerr(isInner);

            AdminWebsit adminWebsit;

            adminWebsit = adminWebsitService.lambdaQuery()
                    .eq(AdminWebsit::getCompanyWechatId, companyWechat.getCompanyWechatId())
                    .eq(AdminWebsit::getWebsitNumber, user.getWebsitId())
                    .one();
            if (Objects.isNull(adminWebsit)) {
                adminWebsit = adminWebsitService.lambdaQuery()
                        .eq(AdminWebsit::getCompanyWechatId, companyWechat.getCompanyWechatId())
                        .orderByAsc(AdminWebsit::getCreateTime)
                        .last("limit 1")
                        .one();
            }

            user.setWebsitId(adminWebsit.getWebsitId() + "");
            user.setWebsitName(adminWebsit.getName());
            if (!isInner) {
                if (StringUtils.isBlank(user.getWorkUserId())) {
                    throw new RemoteServiceException(user.getMobile() + "账号必填");
                }
                if (StringUtils.isBlank(user.getWorkName())) {
                    throw new RemoteServiceException(user.getMobile() + "姓名必填");
                }
                if (StringUtils.isBlank(user.getPosition())) {
                    throw new RemoteServiceException(user.getMobile() + "职位必填");
                }
                phoneList.add(user);
            } else {
                innerPhoneList.add(user);
            }
        }

        // 检查业务员手机号是否有重复的
        final Map<String, List<User>> UserGroupMap = phoneList.stream().collect(Collectors.groupingBy(User::getMobile));
        for (Map.Entry<String, List<User>> entry : UserGroupMap.entrySet()) {
            if (entry.getValue().size() > 1) {
                throw new RemoteServiceException("文档“业务员”有重复手机号 " + entry.getKey());
            }
        }

        List<User> userList = userService.lambdaQuery()
                .in(User::getMobile, phoneList.stream()
                        .map(User::getMobile)
                        .collect(Collectors.toList()))
                .eq(User::getCompanyWechatId, loginCompanyWechatId)
                .eq(User::getType, UserTypeEnum.GENERAL)
                .eq(User::getStatus, true)
                .list();

        // 变成业务员
        final List<String> userIds = userList.stream().map(User::getUserId).collect(Collectors.toList());
        List<User> upUserList = new ArrayList<>();

        if (CollectionUtil.isNotEmpty(userIds)) {
            this.batchChangeServiceUser(userIds, true);

            // 更新会员资料
            final Map<String, User> importUserMap = phoneList.stream().collect(Collectors.toMap(User::getMobile, Function.identity()));
            for (User user : userList) {
                if (importUserMap.containsKey(user.getMobile())) {
                    User newUser = new User();
                    User importUser = importUserMap.get(user.getMobile());
                    newUser.setUserId(user.getUserId())
                            .setMobile(user.getMobile())
                            .setWorkUserId(importUser.getWorkUserId())
                            .setWorkName(importUser.getWorkName())
                            .setWorkPhone(user.getMobile())
                            .setWorkCreateTime(new Date())
                            .setPosition(importUser.getPosition())
                            .setWebsitId(importUser.getWebsitId())
                            .setWebsitName(importUser.getWebsitName());
                    upUserList.add(newUser);

                }
            }
            if (CollectionUtil.isNotEmpty(upUserList)) {
                userService.saveOrUpdateBatch(upUserList);
            }

        }

        // 获取已存在记录
        final List<UserServiceWait> existWaits = userServiceWaitService.lambdaQuery()
                .eq(UserServiceWait::getCompanyWechatId, loginCompanyWechatId)
                .in(UserServiceWait::getMobile, phoneList.stream().map(User::getMobile).collect(Collectors.toList()))
                .list();
        Map<String, UserServiceWait> waitMap = existWaits.stream().collect(Collectors.toMap(UserServiceWait::getMobile, Function.identity()));

        // 遍历产生待处理记录
        List<UserServiceWait> waitList = new ArrayList<>();
        final Map<String, User> upUserMap = upUserList.stream().collect(Collectors.toMap(User::getMobile, Function.identity()));
        final DateTime curDate = DateUtil.date();
        for (User waitUser : phoneList) {
            if (waitMap.containsKey(waitUser.getMobile())) {
                // 如果存在记录忽略生成此手机号对象
                waitList.add(waitMap.get(waitUser.getMobile()));
                continue;
            }
            UserServiceWait wait = new UserServiceWait();
            wait.setMobile(waitUser.getMobile())
                    .setWebsitId(waitUser.getWebsitId())
                    .setWebsitName(waitUser.getWebsitName())
                    .setPosition(waitUser.getPosition())
                    .setName(waitUser.getWorkName())
                    .setWorkerId(waitUser.getWorkUserId())
                    .setCreateTime(curDate)
                    .setCreateBy(adminUser.getNickName())
                    .setCompanyWechatId(loginCompanyWechatId)
                    .setCompanyName(adminUser.getLoginCompanyName());
            waitList.add(wait);
        }
        // 待处理记录不为空批量插入
        if (CollectionUtil.isNotEmpty(waitList)) {
            for (UserServiceWait serviceWait : waitList) {
                // 本次更新用户资料在历史等待处理记录匹配到手机号
                if (upUserMap.containsKey(serviceWait.getMobile())) {
                    final User upUser = upUserMap.get(serviceWait.getMobile());
                    serviceWait.setUserId(upUser.getUserId())
                            .setIsBind(true)
                            .setUpdateTime(new Date());
                }
            }

            userServiceWaitService.saveOrUpdateBatch(waitList);
        }

        if (CollectionUtil.isNotEmpty(innerPhoneList)) {
            // 升级为内部业务员
            userService.lambdaUpdate()
                    .set(User::getInnerr, true)
                    .in(User::getMobile, innerPhoneList.stream()
                            .map(User::getMobile)
                            .collect(Collectors.toList()))
                    .eq(User::getCompanyWechatId, loginCompanyWechatId)
                    .eq(User::getType, UserTypeEnum.SERVICE)
                    .eq(User::getStatus, true)
                    .update();
        }
    }


    /**
     * 设置/取消团长
     */
    @Transactional
    public void groupleader(String userId, Boolean promotionGroupLeader) {
        User user = userService.getById(userId);
        if (user == null) {
            throw new RemoteServiceException("用户不存在");
        }
        user.setUserId(userId);
        user.setPromotionGroupLeader(promotionGroupLeader);
        user.setPromotionGroupUserId(userId);
        user.updateById();
        //添加到所有团购活动里
        if (promotionGroupLeader) {

            //    promotionGroupLogic.addGroupUser(user);
        } else {
            promotionGroupLogic.deleteGroupUser(user);
        }
    }

    /**
     * 用户详情
     */
    public UserWxBean detail(String userId) {
        User user = userService.getById(userId);

        UserWxBean userWxBean = new UserWxBean();
        BeanUtils.copyProperties(user, userWxBean);

        AdminCompanyWechat adminCompanyWechat = commonLogic.getAdminCompanyWechat(user.getCompanyWechatId());

        //查询客户数
        Long count = userService.lambdaQuery().eq(User::getServiceId, userId).count();

        userWxBean.setCustomerNums(Math.toIntExact(count));
        userWxBean.setTotalAmount(orderShareLogic.totalAmount(userId));

        if (StringUtils.isNotEmpty(user.getServiceId())) {
            User service = userService.getById(user.getServiceId());
            if (service != null) {
                userWxBean.setServiceName(service.getWorkName());
                userWxBean.setServicePhone(service.getWorkPhone());
            }
        }

        return userWxBean;
    }


    /**
     * 修改用户昵称
     */
    public void updateUserNickName(String userId, String newNickName) {

        userService.lambdaUpdate().set(User::getNickName, newNickName).eq(User::getUserId, userId).update();

    }

    /**
     * 用户地址
     */
    public IPage<UserAddress> addressList(String userId, Integer pageNo, Integer pageSize) {
        IPage<UserAddress> page = userAddressService.lambdaQuery().eq(UserAddress::getUserId, userId)
                .orderByDesc(UserAddress::getCreateTime)
                .page(new Page<>(pageNo, pageSize));
        return page;
    }

    /**
     * 我的收藏
     */
    public IPage<Goods> goodsFavoriteList(String userId, Integer pageNo, Integer pageSize) {
        IPage<Goods> goodsIPage = customGoodsMapper.myGoodsFavorite(new Page(pageNo, pageSize), userId);
        return goodsIPage;
    }

    /**
     * 我的浏览记录
     */
    public IPage<GoodsVisit> goodsVisitList(String userId, Integer pageNo, Integer pageSize) {
        return goodsVisitService.lambdaQuery().eq(GoodsVisit::getUserId, userId).page(new Page<>(pageNo, pageSize));
    }

    /**
     * 我的优惠券
     */
    public IPage<UserCoupon> couponList(String userId, Integer pageNo, Integer pageSize) {
        return couponLogic.userCouponList(userId, pageNo, pageSize);
    }

    /**
     * 用户降级
     */
    public void demotion(List<String> workUserIds) {
        List<User> list = userService.lambdaQuery().in(User::getWorkUserId, workUserIds).list();
        if (list.size() == 0) {
            return;
        }
        List<String> userIds = list.stream().map(User::getUserId).collect(Collectors.toList());
        //1.将用户设为普通用户
        userService.lambdaUpdate()
                .set(User::getType, UserTypeEnum.GENERAL.toString())
                .in(User::getWorkUserId, workUserIds).update();

        //2.禁止用户分享优惠券
        couponLogic.updateUserCouponTransfer(userIds, false);
    }


    /**
     * 用户升级
     */
    public void userLevelUp(List<String> workUserIds) {
        List<User> list = userService.lambdaQuery().in(User::getWorkUserId, workUserIds).list();
        if (list.size() == 0) {
            return;
        }
        List<String> userIds = list.stream().map(User::getUserId).collect(Collectors.toList());
        //1.将用户设为普通用户
        userService.lambdaUpdate()
                .set(User::getType, UserTypeEnum.SERVICE.toString())
                .in(User::getWorkUserId, workUserIds)
                .update();

        //2.禁止用户分享优惠券
        couponLogic.updateUserCouponTransfer(userIds, true);
    }


    public User getById(String userId) {
        return userService.getById(userId);
    }

    public List<User> getById(List<String> userIds) {
        if (CollectionUtils.isEmpty(userIds)) {
            return new ArrayList<>();
        }
        return userService.lambdaQuery().in(User::getUserId, userIds).orderByDesc(User::getCreateTime).list();
    }

    public List<User> getByMobile(List<String> mobiles) {
        if (CollectionUtils.isEmpty(mobiles)) {
            return new ArrayList<>();
        }
        return userService.lambdaQuery().in(User::getMobile, mobiles).orderByDesc(User::getCreateTime).list();
    }

    @Transactional
    public void examineApply(String id, String websitId, String websitName, String workUserId, String workName, String position, String mobile, Boolean isOK) {
        AdminUserCom adminUser = commonLogic.getAdminUser();
        UserServiceApply apply = userServiceApplyService.getById(id);

        if (StringUtils.isBlank(workUserId)) {
            throw new RemoteServiceException("账号必填");
        }
        if (StringUtils.isBlank(workName)) {
            throw new RemoteServiceException("姓名必填");
        }
        if (StringUtils.isBlank(position)) {
            throw new RemoteServiceException("职位必填");
        }
        if (StringUtils.isBlank(mobile)) {
            throw new RemoteServiceException("手机号必填");
        }
        if (StringUtils.isBlank(websitId)) {
            throw new RemoteServiceException("网点编号必填");
        }
        if (StringUtils.isBlank(websitName)) {
            throw new RemoteServiceException("网点名称必填");
        }

        apply.setStatus(isOK ? "OK" : "FAIL")
                .setUpdateTime(DateUtil.date())
                .setUpdateBy(adminUser.getNickName())
                .updateById();

        // 通过更新用户为业务员并注入手机号
        if (isOK) {
            final List<String> userIds = Collections.singletonList(apply.getUserId());
            this.batchChangeServiceUser(userIds, true);

            userService.lambdaUpdate()
                    .set(User::getWorkUserId, workUserId)
                    .set(User::getWebsitId, websitId)
                    .set(User::getWebsitName, websitName)
                    .set(User::getWorkName, workName)
                    .set(User::getPosition, position)
                    .set(User::getWorkPhone, mobile)
                    .set(User::getWorkCreateTime, new Date())
                    .set(User::getMobile, apply.getMobile())
                    .eq(User::getUserId, apply.getUserId())
                    .update();
        }
    }

    public IPage<UserServiceWait> importMemberList(String websitId, String websitName, String workUserId, String workName,
                                                   String position, String mobile, Integer pageNum, Integer pageSize) {
        AdminUserCom adminUser = commonLogic.getAdminUser();
        return userServiceWaitService.lambdaQuery()
                .eq(UserServiceWait::getCompanyWechatId, adminUser.getLoginCompanyWechatId())
                .like(StringUtils.isNotBlank(websitId), UserServiceWait::getWebsitId, websitId)
                .like(StringUtils.isNotBlank(websitName), UserServiceWait::getWebsitName, websitName)
                .like(StringUtils.isNotBlank(workUserId), UserServiceWait::getWorkerId, workUserId)
                .like(StringUtils.isNotBlank(workName), UserServiceWait::getName, workName)
                .like(StringUtils.isNotBlank(position), UserServiceWait::getPosition, position)
                .like(StringUtils.isNotBlank(mobile), UserServiceWait::getMobile, mobile)
                .orderByDesc(UserServiceWait::getCreateTime)
                .page(new Page<>(pageNum, pageSize));
    }

    public IPage<UserApplyBean> applyPage(HttpServletRequest request, String keyword, String status, Integer pageNum, Integer pageSize) {
        //获取当前登录企业微信id
        AdminUserCom adminUser = commonLogic.getAdminUser(request);
        List<String> companyWechatIds = adminUser.getCompanyWechatIds();
        IPage<UserApplyBean> page = customMemberMapper.queryServiceApplyList(new Page<>(pageNum, pageSize), companyWechatIds, keyword, status);
        return page;
    }

    /**
     * 绑定业务员
     */
    public void bind(String userId, String serviceId) {
        User user = userService.getById(userId);
        User service = userService.getById(serviceId);
        //绑定业务员
        if (user.getType().equals(UserTypeEnum.GENERAL.toString()) && service.getType().equals(UserTypeEnum.SERVICE.toString())) {
            user.setServiceId(serviceId);
            user.setServiceTime(new Date());
        }

        //判断是否为团长
        if (!user.getPromotionGroupLeader()) {
            if (service.getPromotionGroupLeader()) {
                user.setPromotionGroupUserId(serviceId);
            } else {
                String promotionGroupUserId = service.getPromotionGroupUserId();
                user.setPromotionGroupUserId(promotionGroupUserId);
            }
        }

        user.updateById();
    }

    /**
     * 获取绑定业务员小程序码
     */
    public String bindQrCode(HttpServletRequest request) throws RemoteServiceException {
        CurrentCompanyWechat currentCompanyWechat = commonLogic.getCurrentCompanyWechat(request);
        String userId = CommonUtils.getUserId(request);
        User user = userService.lambdaQuery()
                .eq(User::getUserId, userId)
                .eq(User::getStatus, true)
                .one();
        if (user == null) {
            throw new RemoteServiceException("用户不存在");
        }
        if (StringUtils.isNotEmpty(user.getBindQrcode())) {
            return user.getBindQrcode();
        }
        String qrcode = null;
        try {
            qrcode = wechatLogic.getQrcode(QrCodeEnum.BIND.toString().toLowerCase(Locale.ROOT),
                    user.getWorkUserId(), user.getUserId(), currentCompanyWechat.getCompanyWechatId());
            if (StringUtils.isNotEmpty(qrcode)) {
                user.setBindQrcode(qrcode);
                userService.updateById(user);
            }
        } catch (Exception e) {
            log.error("获取绑定业务员小程序码失败", e);
        }
        return qrcode;

    }

    public RankBean rank(HttpServletRequest request) throws RemoteServiceException {
        CurrentCompanyWechat currentCompanyWechat = commonLogic.getCurrentCompanyWechat(request);
        String userId = CommonUtils.getUserId(request);
        User user = userService.lambdaQuery()
                .eq(User::getUserId, userId)
                .eq(User::getStatus, true)
                .one();
        if (user == null || StringUtils.isEmpty(user.getWorkUserId())) {
            throw new RemoteServiceException("业务员不存在");
        }

        Date now = new Date();
        Date endTime = DateUtil.beginOfMonth(now);
        Date startTime = DateUtil.beginOfMonth(DateUtil.lastMonth());
        Date endTimeDay = DateUtil.offsetDay(now, -1);
        RankBean rankBean = new RankBean();
        List<RankItemBean> rankItemBeans = orderShareRankMapper.queryRankList(null, endTime, endTimeDay, currentCompanyWechat.getCompanyWechatId());
        List<RankItemBean> myRanks = null;
        if (rankItemBeans != null && rankItemBeans.size() > 0) {
            rankBean.setRankList(rankItemBeans.size() > 10 ? rankItemBeans.subList(0, 10) : rankItemBeans);
            //此处的workUserId就是userId
            myRanks = rankItemBeans.stream()
                    .filter(o -> o.getWorkUserId() != null && o.getWorkUserId().equals(user.getUserId()))
                    .limit(1)
                    .collect(Collectors.toList());
        }
        RankItemBean myrank = myRanks != null && myRanks.size() > 0 ? myRanks.get(0) : orderShareRankMapper.queryWorkerInfo(user.getUserId(), currentCompanyWechat.getCompanyWechatId());
        rankBean.setMyRank(myrank);

        BigDecimal previousIncome = new BigDecimal("0.00");
        BigDecimal currentIncome = new BigDecimal("0.00");
        List<RankItemBean> previous = orderShareRankMapper.queryRankList(user.getUserId(), startTime, endTime, currentCompanyWechat.getCompanyWechatId());
        List<RankItemBean> current = orderShareRankMapper.queryRankList(user.getUserId(), endTime, endTimeDay, currentCompanyWechat.getCompanyWechatId());
        if (previous != null && previous.size() > 0) {
            previousIncome = previous.get(0).getTotal();
        }
        if (current != null && current.size() > 0) {
            currentIncome = current.get(0).getTotal();
        }

        rankBean.setUpOrDown(currentIncome.compareTo(previousIncome) > 0 ? true : false);
        rankBean.setIncome(currentIncome);
        Calendar date = Calendar.getInstance();
        date.setTime(endTimeDay);
        //date.add(Calendar.SECOND, -1);

        SimpleDateFormat simpleDateFormat = new SimpleDateFormat("yyyy-MM-dd HH:mm");
        rankBean.setStartPeriod(simpleDateFormat.format(endTime));
        rankBean.setEndPeriod(simpleDateFormat.format(date.getTime()));
        return rankBean;
    }

    /**
     * 微信授权
     *
     * @param code
     * @param serviceId
     * @return
     * @throws RemoteServiceException
     * @throws InterruptedException
     */
    @Transactional
    public UserWxBean addUser(String code, String serviceId, HttpServletRequest request) throws RemoteServiceException, InterruptedException {
        CurrentCompanyWechat currentCompanyWechat = commonLogic.getCurrentCompanyWechat(request);
        final WechatOpenBean wechatOpenBean = wechatLogic.authToken(code, currentCompanyWechat.getSubAppId());

        log.info("【微信授权】：{}", JSONUtil.toJsonStr(wechatOpenBean));

        String key = Constant.RedisPrefix.LOCK_AUTH + ":" + wechatOpenBean.getOpenid();
        Lock obtain = redisLockRegistry.obtain(key);
        if (!obtain.tryLock(2, TimeUnit.SECONDS)) {
            log.error("请勿重复授权");
            return null;
        }
        try {
            //如果用户不存在则新注册
            User user = userService.lambdaQuery()
                    .eq(User::getOpenId, wechatOpenBean.getOpenid())
                    .eq(User::getCompanyWechatId, currentCompanyWechat.getCompanyWechatId())
                    .one();
            if (user == null) {
                user = new User();
                user.setLastLoginTime(new Date());
                user.setNickName("微信用户_" + IdUtil.objectId().substring(16, 24));
                user.setOpenId(wechatOpenBean.getOpenid());
                user.setSessionKey(wechatOpenBean.getSessionKey());
                user.setServiceId(serviceId);
                user.setServiceTime(new Date());
                user.setLastLoginTime(new Date());
                user.setCreateTime(new Date());
                user.setCompanyWechatId(currentCompanyWechat.getCompanyWechatId());
                user.setCompanyName(currentCompanyWechat.getCompanyName());
                userService.save(user);
                log.info("【注册成为会员】：{}", user);
                return this.supplyUser(user);
            }

            //如果用户存在，则判断是否为业务员，并更新相关信息
            if (!user.getStatus()) {
                throw new RemoteServiceException("用户已被冻结");
            }
            //判断该手机号是否为业务员
            user = userToService(user, currentCompanyWechat);

            user.setUnionId(wechatOpenBean.getUnionid());
            user.setSessionKey(wechatOpenBean.getSessionKey());
            user.setLastLoginTime(new Date());
            if (StringUtils.isEmpty(user.getServiceId()) && StringUtils.isNotEmpty(serviceId)) {
                user.setServiceId(serviceId);
                user.setServiceTime(new Date());
            }
            user.setOpenId(wechatOpenBean.getOpenid());
            user.updateById();
            return this.supplyUser(user);
        } finally {
            obtain.unlock();
        }

    }

    /**
     * 返回小程序用户登录需要的信息
     *
     * @param user
     * @return
     */
    private UserWxBean supplyUser(User user) {
        UserWxBean userWxBean = new UserWxBean();
        String token = JwtUtils.generateToken(user.getUserId(), user.getOpenId(), user.getNickName(), null, Constant.MINIAPP_API);
        BeanUtils.copyProperties(user, userWxBean);
        userWxBean.setToken(token);
        userWxBean.setShoppingCartNums(shoppingCartLogic.total(userWxBean.getUserId()));
        //存放到redis
        redisUtil.set(Constant.RedisPrefix.TOKEN_MP_WX + userWxBean.getToken(), user, 2 * 60 * 60 * 1000);
        AdminWebsit adminWebsit = adminWebsitService.lambdaQuery()
                .eq(AdminWebsit::getCompanyWechatId, user.getCompanyWechatId())
                .eq(AdminWebsit::getWebsitId, user.getWebsitId())
                .one();
        if (adminWebsit != null) {
            userWxBean.setWebsitNumber(adminWebsit.getWebsitNumber());
            userWxBean.setWebsitName(adminWebsit.getName());
        }
        if (StringUtils.isNotBlank(user.getServiceId())) {
            final User serviceUser = userService.getById(user.getServiceId());
            ServiceWorkerBean bean = new ServiceWorkerBean();
            if (StringUtils.isNotBlank(serviceUser.getWebsitId())) {
                bean.setWebsitId(Long.parseLong(serviceUser.getWebsitId()));
            }
            bean.setWebsit(serviceUser.getWebsitName());
            userWxBean.setServiceWorkerBean(bean);
        }
        return userWxBean;
    }

    public User userToService(User user, CurrentCompanyWechat currentCompanyWechat) {
        //判断该手机号是否为业务员
        if (StringUtils.isNotEmpty(user.getMobile())) {
            UserServiceWait waitUser = userServiceWaitService.lambdaQuery()
                    .eq(UserServiceWait::getMobile, user.getMobile())
                    .eq(UserServiceWait::getCompanyWechatId, user.getCompanyWechatId())
                    .one();

            if (Objects.nonNull(waitUser)) {
                Long count = userService.lambdaQuery()
                        .eq(User::getMobile, user.getMobile())
                        .eq(User::getType, UserTypeEnum.SERVICE.toString())
                        .eq(User::getCompanyWechatId, user.getCompanyWechatId())
                        .count();
                if (count < 1) {
                    user.setType(UserTypeEnum.SERVICE.toString());
                    user.setWorkName(waitUser.getName());
                    user.setWorkUserId(waitUser.getWorkerId());
                    user.setPosition(waitUser.getPosition());
                    user.setWorkPhone(waitUser.getMobile());
                    user.setWebsitId(waitUser.getWebsitId());
                    user.setWebsitName(waitUser.getWebsitName());
                    user.setCompanyWechatId(currentCompanyWechat.getCompanyWechatId());
                    user.setCompanyName(currentCompanyWechat.getCompanyName());

                    waitUser.setUserId(user.getUserId())
                            .setIsBind(true)
                            .setUpdateTime(new Date())
                            .updateById();
                }
            }
        }
        return user;
    }

    /**
     * 新增用户访问记录
     */
    public void addVisit(HttpServletRequest request, String userId) {
        try {
            CurrentCompanyWechat currentCompanyWechat = commonLogic.getCurrentCompanyWechat(request);
            UserVisit userVisit = new UserVisit();
            userVisit.setUserId(userId);
            userVisit.setVisitTime(new Date());
            userVisit.setCompanyWechatId(currentCompanyWechat.getCurrentCompanyWechatId());
            userVisit.setCompanyName(currentCompanyWechat.getCurrentCompanyName());
            userVisit.insert();
        } catch (Exception e) {
            log.error("新增用户访问记录失败", e);
        }
    }

    //手机号码授权
    @Transactional
    public UserWxBean getWxMobile(String code, String userId, HttpServletRequest request) throws Exception {
        CurrentCompanyWechat currentCompanyWechat = commonLogic.getCurrentCompanyWechat(request);
        UserWxBean userWxBean = new UserWxBean();
        User user = this.userService.getById(userId);
        if (null == user) {
            throw new RemoteServiceException("用户信息为空" + userId);
        }
        WxMaService maService = wechatLogic.getMaService(currentCompanyWechat.getCompanyWechatId());
        WxMaPhoneNumberInfo phoneNoInfo = maService.getUserService().getPhoneNoInfo(code);
        String mobile = phoneNoInfo.getPhoneNumber();
        if (StringUtils.isEmpty(mobile)) {
            throw new RemoteServiceException("手机号授权失败");
        }

        List<UserServiceWait> waitList = userServiceWaitService.lambdaQuery()
                .eq(UserServiceWait::getCompanyWechatId, currentCompanyWechat.getCompanyWechatId())
                .eq(UserServiceWait::getMobile, mobile)
                .list();

        //根据手机号查询是否存在企业微信用户
        User workUser = userService.lambdaQuery()
                .eq(User::getMobile, mobile)
                .eq(User::getStatus, true)
                .eq(User::getCompanyWechatId, currentCompanyWechat.getCompanyWechatId())
                .one();
        if (workUser == null) {
            //企业微信用户为空，则直接授权手机号。并根据手机号判断是否存在于微信成员表如果存在则为业务员
            user.setMobile(mobile);
            if (waitList.size() > 0) {
                UserServiceWait wait = waitList.get(0);
                String workUserId = wait.getWorkerId();
                Long count = userService.lambdaQuery()
                        .eq(User::getWorkUserId, workUserId)
                        .eq(User::getType, UserTypeEnum.SERVICE.toString())
                        .eq(User::getCompanyWechatId, currentCompanyWechat.getCompanyWechatId())
                        .count();
                if (count < 1) {
                    user.setType(UserTypeEnum.SERVICE.toString());
                    user.setWorkName(wait.getName());
                    user.setWorkUserId(workUserId);
                    user.setWorkPhone(wait.getMobile());
//                    user.setWorkQrcode(list.get(0).getQrCode());
                    user.setPosition(wait.getPosition());
                    user.setWebsitId(wait.getWebsitId());
                    user.setWebsitName(wait.getWebsitName());
                    user.setWorkCreateTime(new Date());

                    wait.setUserId(user.getUserId())
                            .setIsBind(true)
                            .setUpdateTime(new Date())
                            .updateById();
                }
            }
            user.updateById();

            String token = JwtUtils.generateToken(user.getUserId(), user.getOpenId(), user.getNickName(), null, Constant.MINIAPP_API);
            BeanUtils.copyProperties(user, userWxBean);
            userWxBean.setToken(token);

        } else {
            //企业微信不为空，则将数据绑定到企业微信,并删除授权openid的用户数据
            workUser.setOpenId(user.getOpenId());
            workUser.updateById();
            user.deleteById();

            String token = JwtUtils.generateToken(user.getUserId(), user.getOpenId(), user.getNickName(), null, Constant.MINIAPP_API);
            BeanUtils.copyProperties(workUser, userWxBean);
            userWxBean.setToken(token);
        }
        return userWxBean;
    }

    /**
     * 个人信息
     */
    public UserWxBean userDetail(String userId, HttpServletRequest request) throws RemoteServiceException {
        User user = userService.getById(userId);
        if (user == null) {
            throw new RemoteServiceException(ResponseHelper.ResponseCode_AUTH_ERROR, "请求不合法");
        }
        UserWxBean userWxBean = new UserWxBean();
        BeanUtils.copyProperties(user, userWxBean);
        userWxBean.setShoppingCartNums(shoppingCartLogic.total(userWxBean.getUserId()));
        if (StringUtils.isNotBlank(user.getServiceId())) {
            final User serviceUser = userService.getById(user.getServiceId());
            ServiceWorkerBean bean = new ServiceWorkerBean();
            if (StringUtils.isNotBlank(serviceUser.getWebsitId())) {
                bean.setWebsitId(Long.parseLong(serviceUser.getWebsitId()));
            }
            bean.setWebsit(serviceUser.getWebsitName());
            userWxBean.setServiceWorkerBean(bean);
        }
        return userWxBean;
    }

    //保存用户信息
    public User saveUserInfo(String userId, String nickName, String sex, String avatarUrl, String country, String province, String city) {
        User user = new User();
        user.setUserId(userId);
        user.setNickName(nickName);
        user.setSex(sex);
        user.setAvatar(avatarUrl);
        user.setCountry(country);
        user.setProvince(province);
        user.setCity(city);
        user.updateById();
        return user;
    }

    /**
     * 我的收益
     */
    public MyProfitBean myProfit(String userId) {
        User user = userService.getById(userId);
        if (user == null) {
            return null;
        }
        MyProfitBean myProfitBean = customMemberMapper.queryProfitSum(user.getWorkUserId(), user.getUserId());
        if (myProfitBean != null) {
            myProfitBean.setTotalAmount(myProfitBean.getPaidAmount().add(myProfitBean.getWaitingAmount()));
        } else {
            myProfitBean = new MyProfitBean();
            BigDecimal zero = new BigDecimal("0.00");
            myProfitBean.setPaidAmount(zero);
            myProfitBean.setWaitingAmount(zero);
            myProfitBean.setTotalAmount(zero);
        }
        return myProfitBean;
    }


    public IPage<OrderShare> myProfitList(String userId, String status, Integer pageNum, Integer pageSize) {
        User user = userService.getById(userId);
        if (user == null) {
            return null;
        }
        return orderShareService.lambdaQuery()
                .eq(OrderShare::getOpenId, user.getOpenId())
                .ne(OrderShare::getWorkUserId, "")
                .eq(StringUtils.isNotEmpty(status), OrderShare::getStatus, status)
                .eq(OrderShare::getCompanyWechatId, user.getCompanyWechatId())
                .orderByDesc(OrderShare::getCreateTime)
                .page(new Page<>(pageNum, pageSize));
    }

    /**
     * 总收益
     */
    public BigDecimal totalProfit(String userId) {
        User user = userService.getById(userId);
        if (user == null || StringUtils.isEmpty(user.getWorkUserId())) {
            return null;
        }
        List<OrderShare> list = orderShareService.lambdaQuery()
                .eq(OrderShare::getOpenId, user.getOpenId())
                .eq(OrderShare::getCompanyWechatId, user.getCompanyWechatId())
                .and(v -> v.like(OrderShare::getWorkUserId, user.getWorkUserId()).or().like(OrderShare::getWorkUserId, user.getUserId()))
                .eq(OrderShare::getStatus, OrderShareStatusEnum.OVER.toString())
                .list();
        if (list.size() == 0) {
            return new BigDecimal(0);
        }
        return list.stream().map(OrderShare::getAmount).reduce(BigDecimal::add).orElse(new BigDecimal(0));
    }

    /**
     * 我的客户
     */
    public IPage<User> customerList(String userId, Integer pageNum, Integer pageSize) {
        IPage<User> page = userService.lambdaQuery().eq(User::getServiceId, userId).page(new Page<>(pageNum, pageSize));
        return page;
    }

    /**
     * 我的经理
     */
    public User parentUser(String userId) {
        User user = userService.getById(userId);
        if (StringUtils.isNotEmpty(user.getServiceId())) {
            User parentUser = userService.getById(user.getServiceId());
            return parentUser;
        }
        return null;
    }

    public String renewal(String token) {
        final Object o = redisUtil.get(Constant.RedisPrefix.TOKEN_MP_WX + token);
        if (Objects.isNull(o)) {
            throw new RemoteServiceException(ResponseHelper.ResponseCode_AUTH_ERROR, "登录过期");
        }

        User user = (User) o;
        final User user1 = userService.getById(user.getUserId());

        String newToken = JwtUtils.refreshToken(token, Constant.MINIAPP_API);

        //存放到redis
        redisUtil.set(Constant.RedisPrefix.TOKEN_MP_WX + newToken, user1, 2 * 60 * 60 * 1000);
        return newToken;
    }
}
