package com.gree.mall.contest.logic.user;

import cn.hutool.core.date.DateUtil;
import com.gree.mall.contest.bean.user.CurrentCompanyWechat;
import com.gree.mall.contest.enums.UserTypeEnum;
import com.gree.mall.contest.exception.RemoteServiceException;
import com.gree.mall.contest.logic.SMSLogic;
import com.gree.mall.contest.logic.common.CommonLogic;
import com.gree.mall.contest.plus.entity.AdminWebsit;
import com.gree.mall.contest.plus.entity.User;
import com.gree.mall.contest.plus.entity.UserServiceApply;
import com.gree.mall.contest.plus.service.AdminWebsitService;
import com.gree.mall.contest.plus.service.UserService;
import lombok.RequiredArgsConstructor;
import lombok.extern.slf4j.Slf4j;
import org.springframework.stereotype.Service;

import javax.servlet.http.HttpServletRequest;
import java.util.List;
import java.util.regex.Matcher;
import java.util.regex.Pattern;

@Slf4j
@Service
@RequiredArgsConstructor
public class WorkerLogic {

    private final SMSLogic smsLogic;
    private final CommonLogic commonLogic;
    private final UserService userService;
    private final AdminWebsitService adminWebsitService;

    public void sendSms(String phone, String key, String vrifyCode, HttpServletRequest request) throws Exception {
        if (!this.isPhoneLegal(phone)) {
            throw new RemoteServiceException("手机号不正确");
        }

        CurrentCompanyWechat currentCompanyWechat = commonLogic.getCurrentCompanyWechat(request);

        final Long count = userService.lambdaQuery()
                .eq(User::getCompanyWechatId, currentCompanyWechat.getCompanyWechatId())
                .eq(User::getType, UserTypeEnum.SERVICE.toString())
                .eq(User::getMobile, phone)
                .count();

        if (count > 0) {
            throw new RemoteServiceException("手机号已被绑定");
        }
        smsLogic.sendSms(phone, key, vrifyCode, "APPLY_WORKER");
    }

    private boolean isPhoneLegal(String phone) {
        Pattern CHINA_PATTERN = Pattern.compile("^((13[0-9])|(14[0,1,4-9])|(15[0-3,5-9])|(16[2,5,6,7])|(17[0-8])|(18[0-9])|(19[0-3,5-9]))\\d{8}$");
        Matcher m = CHINA_PATTERN.matcher(phone);
        return m.matches();
    }


    public void bind(HttpServletRequest request, String phone, String code) throws RemoteServiceException {
        smsLogic.checkSmsCode(phone, code, "APPLY_WORKER");
        CurrentCompanyWechat currentCompanyWechat = commonLogic.getCurrentCompanyWechat(request);

        final Long count = userService.lambdaQuery()
                .eq(User::getCompanyWechatId, currentCompanyWechat.getCompanyWechatId())
                .eq(User::getType, UserTypeEnum.SERVICE.toString())
                .eq(User::getMobile, phone)
                .count();

        if (count > 0) {
            throw new RemoteServiceException("手机号已被绑定");
        }

        final User user = userService.getById(currentCompanyWechat.getUserId());

        if (!user.getType().equals(UserTypeEnum.GENERAL.toString())) {
            throw new RemoteServiceException("当前用户已成为”业务员“");
        }

        UserServiceApply apply = new UserServiceApply();
        apply.setUserId(user.getUserId())
                .setMobile(phone)
                .setCreateTime(DateUtil.date())
                .setCompanyWechatId(user.getCompanyWechatId())
                .setCompanyName(user.getCompanyName())
                .insert();

    }

    public List<AdminWebsit> websitList(HttpServletRequest request) {
        CurrentCompanyWechat currentCompanyWechat = commonLogic.getCurrentCompanyWechat(request);
        final List<AdminWebsit> websitList = adminWebsitService.lambdaQuery()
                .eq(AdminWebsit::getCompanyWechatId, currentCompanyWechat.getCompanyWechatId())
                .orderByAsc(AdminWebsit::getCreateTime)
                .list();

        return websitList;
    }
}
